@@ -17,11 +17,6 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-
-////
-//// Includes
-////
-
 #if !defined(_MSC_VER)
 
 #  include <sys/time.h>
@@ -55,83 +50,18 @@
 
 using namespace std;
 
-/// Version number. If this is left empty, the current date (in the format
-/// YYMMDD) is used as a version number.
+/// Version number. If EngineVersion is left empty, then AppTag plus
+/// current date (in the format YYMMDD) is used as a version number.
 
-static const string EngineVersion = "2.0.1";
 static const string AppName = "Stockfish";
+static const string EngineVersion = "2.1.1";
 static const string AppTag  = "";
 
 
-////
-//// Variables
-////
-
-static uint64_t dbg_cnt0 = 0;
-static uint64_t dbg_cnt1 = 0;
-
-bool dbg_show_mean = false;
-bool dbg_show_hit_rate = false;
-
-
-////
-//// Functions
-////
-
-void dbg_hit_on(bool b) {
-
-    assert(!dbg_show_mean);
-    dbg_show_hit_rate = true;
-    dbg_cnt0++;
-    if (b)
-        dbg_cnt1++;
-}
-
-void dbg_hit_on_c(bool c, bool b) {
-
-    if (c)
-        dbg_hit_on(b);
-}
-
-void dbg_before() {
-
-    assert(!dbg_show_mean);
-    dbg_show_hit_rate = true;
-    dbg_cnt0++;
-}
-
-void dbg_after() {
-
-    assert(!dbg_show_mean);
-    dbg_show_hit_rate = true;
-    dbg_cnt1++;
-}
-
-void dbg_mean_of(int v) {
-
-    assert(!dbg_show_hit_rate);
-    dbg_show_mean = true;
-    dbg_cnt0++;
-    dbg_cnt1 += v;
-}
-
-void dbg_print_hit_rate() {
-
-  cout << "Total " << dbg_cnt0 << " Hit " << dbg_cnt1
-       << " hit rate (%) " << (dbg_cnt1*100)/(dbg_cnt0 ? dbg_cnt0 : 1) << endl;
-}
-
-void dbg_print_mean() {
-
-  cout << "Total " << dbg_cnt0 << " Mean "
-       << (float)dbg_cnt1 / (dbg_cnt0 ? dbg_cnt0 : 1) << endl;
-}
-
-
 /// engine_name() returns the full name of the current Stockfish version.
 /// This will be either "Stockfish YYMMDD" (where YYMMDD is the date when
 /// the program was compiled) or "Stockfish <version number>", depending
-/// on whether the constant EngineVersion (defined in misc.h) is empty.
+/// on whether the constant EngineVersion is empty.
 
 const string engine_name() {
 
@@ -155,62 +85,100 @@ const string engine_name() {
 }
 
 
-/// get_system_time() returns the current system time, measured in
-/// milliseconds.
+/// Our brave developers! Required by UCI
+
+const string engine_authors() {
+
+  return "Tord Romstad, Marco Costalba and Joona Kiiski";
+}
+
+
+/// Debug stuff. Helper functions used mainly for debugging purposes
+
+static uint64_t dbg_hit_cnt0;
+static uint64_t dbg_hit_cnt1;
+static uint64_t dbg_mean_cnt0;
+static uint64_t dbg_mean_cnt1;
+
+void dbg_print_hit_rate() {
+
+  if (dbg_hit_cnt0)
+      cout << "Total " << dbg_hit_cnt0 << " Hit " << dbg_hit_cnt1
+           << " hit rate (%) " << 100 * dbg_hit_cnt1 / dbg_hit_cnt0 << endl;
+}
+
+void dbg_print_mean() {
+
+  if (dbg_mean_cnt0)
+      cout << "Total " << dbg_mean_cnt0 << " Mean "
+           << (float)dbg_mean_cnt1 / dbg_mean_cnt0 << endl;
+}
+
+void dbg_mean_of(int v) {
+
+  dbg_mean_cnt0++;
+  dbg_mean_cnt1 += v;
+}
+
+void dbg_hit_on(bool b) {
+
+  dbg_hit_cnt0++;
+  if (b)
+      dbg_hit_cnt1++;
+}
+
+void dbg_hit_on_c(bool c, bool b) { if (c) dbg_hit_on(b); }
+void dbg_before() { dbg_hit_on(false); }
+void dbg_after()  { dbg_hit_on(true); dbg_hit_cnt0--; }
+
+
+/// get_system_time() returns the current system time, measured in milliseconds
 
 int get_system_time() {
 
 #if defined(_MSC_VER)
-    struct _timeb t;
-    _ftime(&t);
-    return int(t.time*1000 + t.millitm);
+  struct _timeb t;
+  _ftime(&t);
+  return int(t.time * 1000 + t.millitm);
 #else
-    struct timeval t;
-    gettimeofday(&t, NULL);
-    return t.tv_sec*1000 + t.tv_usec/1000;
+  struct timeval t;
+  gettimeofday(&t, NULL);
+  return t.tv_sec * 1000 + t.tv_usec / 1000;
 #endif
 }
 
 
-/// cpu_count() tries to detect the number of CPU cores.
-
-#if !defined(_MSC_VER)
-
-#  if defined(_SC_NPROCESSORS_ONLN)
-int cpu_count() {
-  return Min(sysconf(_SC_NPROCESSORS_ONLN), MAX_THREADS);
-}
-#  elif defined(__hpux)
-int cpu_count() {
-  struct pst_dynamic psd;
-  if (pstat_getdynamic(&psd, sizeof(psd), (size_t)1, 0) == -1)
-      return 1;
-
-  return Min(psd.psd_proc_cnt, MAX_THREADS);
-}
-#  else
-int cpu_count() {
-  return 1;
-}
-#  endif
-
-#else
+/// cpu_count() tries to detect the number of CPU cores
 
 int cpu_count() {
+
+#if defined(_MSC_VER)
   SYSTEM_INFO s;
   GetSystemInfo(&s);
   return Min(s.dwNumberOfProcessors, MAX_THREADS);
-}
+#else
+
+#  if defined(_SC_NPROCESSORS_ONLN)
+  return Min(sysconf(_SC_NPROCESSORS_ONLN), MAX_THREADS);
+#  elif defined(__hpux)
+  struct pst_dynamic psd;
+  if (pstat_getdynamic(&psd, sizeof(psd), (size_t)1, 0) == -1)
+      return 1;
+  return Min(psd.psd_proc_cnt, MAX_THREADS);
+#  else
+  return 1;
+#  endif
 
 #endif
+}
 
 
-/// Check for console input. Original code from Beowulf and Olithink
+/// Check for console input. Original code from Beowulf, Olithink and Greko
 
 #ifndef _WIN32
 
-int data_available()
-{
+int input_available() {
+
   fd_set readfds;
   struct timeval  timeout;
 
@@ -225,43 +193,54 @@ int data_available()
 
 #else
 
-int data_available()
-{
-    static HANDLE inh = NULL;
-    static bool usePipe;
-    INPUT_RECORD rec[256];
-    DWORD dw, recCnt;
+int input_available() {
 
-    if (!inh)
-    {
-        inh = GetStdHandle(STD_INPUT_HANDLE);
-        usePipe = !GetConsoleMode(inh, &dw);
-        if (!usePipe)
-        {
-            SetConsoleMode(inh, dw & ~(ENABLE_MOUSE_INPUT | ENABLE_WINDOW_INPUT));
-            FlushConsoleInputBuffer(inh);
-        }
-    }
+  static HANDLE inh = NULL;
+  static bool usePipe = false;
+  INPUT_RECORD rec[256];
+  DWORD nchars, recCnt;
 
-    // If we're running under XBoard then we can't use PeekConsoleInput() as
-    // the input commands are sent to us directly over the internal pipe.
-    if (usePipe)
-        return PeekNamedPipe(inh, NULL, 0, NULL, &dw, NULL) ? dw : 1;
+  if (!inh)
+  {
+      inh = GetStdHandle(STD_INPUT_HANDLE);
+      if (GetConsoleMode(inh, &nchars))
+      {
+          SetConsoleMode(inh, nchars & ~(ENABLE_MOUSE_INPUT | ENABLE_WINDOW_INPUT));
+          FlushConsoleInputBuffer(inh);
+      } else
+          usePipe = true;
+  }
 
-    // Count the number of unread input records, including keyboard,
-    // mouse, and window-resizing input records.
-    GetNumberOfConsoleInputEvents(inh, &dw);
+  // When using Standard C input functions, also check if there
+  // is anything in the buffer. After a call to such functions,
+  // the input waiting in the pipe will be copied to the buffer,
+  // and the call to PeekNamedPipe can indicate no input available.
+  // Setting stdin to unbuffered was not enough. [from Greko]
+  if (stdin->_cnt > 0)
+      return 1;
 
-    // Read data from console without removing it from the buffer
-    if (dw <= 0 || !PeekConsoleInput(inh, rec, Min(dw, 256), &recCnt))
-        return 0;
+  // When running under a GUI the input commands are sent to us
+  // directly over the internal pipe. If PeekNamedPipe() returns 0
+  // then something went wrong. Probably the parent program exited.
+  // Returning 1 will make the next call to the input function
+  // return EOF, where this should be catched then.
+  if (usePipe)
+      return PeekNamedPipe(inh, NULL, 0, NULL, &nchars, NULL) ? nchars : 1;
 
-    // Search for at least one keyboard event
-    for (DWORD i = 0; i < recCnt; i++)
-        if (rec[i].EventType == KEY_EVENT)
-            return 1;
+  // Count the number of unread input records, including keyboard,
+  // mouse, and window-resizing input records.
+  GetNumberOfConsoleInputEvents(inh, &nchars);
 
-    return 0;
+  // Read data from console without removing it from the buffer
+  if (nchars <= 0 || !PeekConsoleInput(inh, rec, Min(nchars, 256), &recCnt))
+      return 0;
+
+  // Search for at least one keyboard event
+  for (DWORD i = 0; i < recCnt; i++)
+      if (rec[i].EventType == KEY_EVENT)
+          return 1;
+
+  return 0;
 }
 
 #endif
@@ -269,9 +248,11 @@ int data_available()
 
 /// prefetch() preloads the given address in L1/L2 cache. This is a non
 /// blocking function and do not stalls the CPU waiting for data to be
-/// loaded from RAM, that can be very slow.
+/// loaded from memory, that can be quite slow.
 #if defined(NO_PREFETCH)
+
 void prefetch(char*) {}
+
 #else
 
 void prefetch(char* addr) {
@@ -287,4 +268,3 @@ void prefetch(char* addr) {
 }
 
 #endif
-
